% Copyright GIPSA-lab, Grenoble, France
% For all questions contact sophie.cerf@gipsa-lab.fr

global LPPM
global weight

%% OBJECTIVES
% Objective ratio: privacy=weight.utility
weight=3;

%% PROFILER
path_to_experiment_directory='spatial&spatiotemporal';
LPPMs={'GeoI','Promesse'};
dataset={'geolife','mdc','privamov','cab'};
nb_user=[42 144 48 536];
tot_user=sum(nb_user);
epsilonP_down=50;
epsilonP_up=10000;
epsilonG_down=10^(-4);
epsilonG_up=1;

% Definition of configuration parameter values for GeoI ...
epsilonG_read={'1.0','0.501187','0.199526','0.125893','0.1','0.050119','0.019953','0.012589','0.01','0.005012','0.001995','0.001259','0.001','5.01E-4','2.0E-4','1.26E-4','1.0E-4'}; % To read the file names 
epsilonG=[10^0, 10^(-0.3),10^(-0.7),10^(-0.9),10^(-1),10^(-1.3),10^(-1.7),10^(-1.9),10^(-2),10^(-2.3),10^(-2.7),10^(-2.9),10^(-3),10^(-3.3),10^(-3.7),10^(-3.9),10^(-4)]; % Used for computation
% ... and for Promesse
epsilonP_read={'70.0.meters', '100.0.meters', '200.0.meters', '400.0.meters', '700.0.meters', '1000.0.meters', '2000.0.meters', '4000.0.meters', '7000.0.meters', '10000.0.meters'}; % To read the file names 
epsilonP=[70, 100, 200, 400, 700, 1000, 2000, 4000, 7000, 10000]; % Used for computation

% Extracting data from experiement files
% privacy and utility are a vector of size (nb of configuration parameter)*(nb of users)
privacyG=[]; utilityG=[]; privacyP=[]; utilityP=[];
for datasetnb=1:4 % for each dataset
    [privacy, utility]=profiler(LPPMs{1},dataset{datasetnb},nb_user(datasetnb),epsilonG_read,path_to_experiment_directory); 
    privacyG=[privacyG privacy]; utilityG=[utilityG utility];
    [privacy, utility]=profiler(LPPMs{2},dataset{datasetnb},nb_user(datasetnb),epsilonP_read,path_to_experiment_directory);
    privacyP=[privacyP privacy]; utilityP=[utilityP utility];
end

%% MODELER
% init size
apG=zeros(tot_user,1); bpG=zeros(tot_user,1); cpG=zeros(tot_user,1); dpG=zeros(tot_user,1); auG=zeros(tot_user,1); buG=zeros(tot_user,1); cuG=zeros(tot_user,1); duG=zeros(tot_user,1);apP=zeros(tot_user,1); bpP=zeros(tot_user,1); cpP=zeros(tot_user,1); dpP=zeros(tot_user,1); auP=zeros(tot_user,1); buP=zeros(tot_user,1); cuP=zeros(tot_user,1); duP=zeros(tot_user,1);
% Find optimal model parameters 
for i=1:tot_user
    LPPM=LPPMs{1};
    [apG(i),bpG(i),cpG(i),dpG(i),auG(i),buG(i),cuG(i),duG(i)]=modeler(epsilonG,privacyG(:,i),utilityG(:,i)); 
    LPPM=LPPMs{2};
    [apP(i),bpP(i),cpP(i),dpP(i),auP(i),buP(i),cuP(i),duP(i)]=modeler(epsilonP,privacyP(:,i),utilityP(:,i));
 end

%% CONFIGURATOR
% init sizes
epsilon_starG=zeros(tot_user,1); privacy_starG=zeros(tot_user,1); utility_starG=zeros(tot_user,1); epsilon_starP=zeros(tot_user,1); privacy_starP=zeros(tot_user,1); utility_starP=zeros(tot_user,1); 
% Finding best configuration for each LPPM, for each user
for i=1:tot_user
    [epsilon_starG(i),privacy_starG(i),utility_starG(i)]=configurator(apG(i),bpG(i),cpG(i),dpG(i),auG(i),buG(i),cuG(i),duG(i),weight,epsilonG_up, epsilonG_down);
    [epsilon_starP(i),privacy_starP(i),utility_starP(i)]=configurator(apP(i),bpP(i),cpP(i),dpP(i),auP(i),buP(i),cuP(i),duP(i),weight,epsilonP_up, epsilonP_down);
end

% init sizes
epsilon_star=zeros(tot_user,1); privacy_star=zeros(tot_user,1); utility_star=zeros(tot_user,1); LPPM_star=[];
% Choosing the best LPPM by comparing privacy+utility (and check definition
% ranges)
for i=1:tot_user
if (privacy_starG(i)+utility_starG(i))<(privacy_starP(i)+utility_starP(i))
    LPPM_star{i}='Promesse';
    epsilon_star(i)=epsilon_starP(i);
    privacy_star(i)=privacy_starP(i);
    utility_star(i)=utility_starP(i);
else
    LPPM_star{i}='GeoI';
    epsilon_star(i)=epsilon_starG(i);
    privacy_star(i)=privacy_starG(i);
    utility_star(i)=utility_starG(i);
end
end

% LPPM_star, epsilon_star, privacy_star & utility_star are respectivelly
% the LPPM and its configuration chosen by PULP to satisfy objectives, and
% the resulting privacy and utility level achieved.